package handlers

import (
	"net/http"

	"git.linuxforward.com/byom/byom-trends/common"
	"git.linuxforward.com/byom/byom-trends/logger"
	"git.linuxforward.com/byom/byom-trends/services/instagram"
	"git.linuxforward.com/byom/byom-trends/services/tiktok"
	"git.linuxforward.com/byom/byom-trends/services/youtube"
	"git.linuxforward.com/byom/byom-trends/store"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
	"github.com/sirupsen/logrus"
)

type TrendHandler struct {
	store        *store.DataStore
	instagramSvc *instagram.Client
	tiktokSvc    *tiktok.Client
	youtubeSvc   *youtube.Client
	logger       *logrus.Entry
}

func NewTrendHandler(
	store *store.DataStore,
	instagramSvc *instagram.Client,
	tiktokSvc *tiktok.Client,
	youtubeSvc *youtube.Client,
) *TrendHandler {
	return &TrendHandler{
		store:        store,
		instagramSvc: instagramSvc,
		tiktokSvc:    tiktokSvc,
		youtubeSvc:   youtubeSvc,
		logger:       logger.NewLogger("trend-handler"),
	}
}

// AnalyzeProfile analyzes trends for a specific profile
func (h *TrendHandler) AnalyzeProfile(c *gin.Context) {
	profileID, err := uuid.Parse(c.Param("id"))
	if err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid profile ID"})
		return
	}

	// Add profile existence check
	profile, err := h.store.GetSocialProfile(c, profileID)
	if err != nil {
		h.logger.WithError(err).Error("failed to get social profile")
		c.JSON(http.StatusInternalServerError, gin.H{"error": "Failed to get social profile"})
		return
	}
	if profile == nil {
		c.JSON(http.StatusNotFound, gin.H{"error": "Profile not found"})
		return
	}

	// Create a new trend analysis
	analysis := &common.TrendAnalysis{
		ID:        uuid.New(),
		ProfileID: profileID,
		Type:      "social",
	}

	// TODO: Implement actual trend analysis logic
	// This would involve:
	// 1. Fetching social media data
	// 2. Analyzing engagement patterns
	// 3. Identifying trending content
	// 4. Generating insights

	if err := h.store.CreateTrendAnalysis(c, analysis); err != nil {
		h.logger.WithError(err).Error("failed to create trend analysis")
		c.JSON(http.StatusInternalServerError, gin.H{"error": "Failed to create trend analysis"})
		return
	}

	c.JSON(http.StatusCreated, analysis)
}

// GetTrendReport retrieves a specific trend analysis
func (h *TrendHandler) GetTrendReport(c *gin.Context) {
	analysisID, err := uuid.Parse(c.Param("id"))
	if err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": "Invalid analysis ID"})
		return
	}

	analysis, err := h.store.GetTrendAnalysis(c, analysisID)
	if err != nil {
		h.logger.WithError(err).Error("failed to get trend analysis")
		c.JSON(http.StatusInternalServerError, gin.H{"error": "Failed to get trend analysis"})
		return
	}

	if analysis == nil {
		c.JSON(http.StatusNotFound, gin.H{"error": "Trend analysis not found"})
		return
	}

	c.JSON(http.StatusOK, analysis)
}
